"""
OpenTree_get_mrca

Find the Most Recent Common Ancestor (MRCA) of a set of taxa in the Open Tree of Life synthetic p...
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def OpenTree_get_mrca(
    ott_ids: str,
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> Any:
    """
    Find the Most Recent Common Ancestor (MRCA) of a set of taxa in the Open Tree of Life synthetic p...

    Parameters
    ----------
    ott_ids : str
        Comma-separated OTT IDs (at least 2). Use OpenTree_match_names to get IDs. Ex...
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    Any
    """
    # Handle mutable defaults to avoid B006 linting error

    # Strip None values so optional parameters don't trigger schema validation errors
    _args = {k: v for k, v in {"ott_ids": ott_ids}.items() if v is not None}
    return get_shared_client().run_one_function(
        {
            "name": "OpenTree_get_mrca",
            "arguments": _args,
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["OpenTree_get_mrca"]



"""
openalex_search_authors

Search OpenAlex authors via the /authors endpoint. Use this to discover Author IDs (A...) and the...
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def openalex_search_authors(
    search: str,
    per_page: Optional[int] = 5,
    page: Optional[int] = 1,
    mailto: Optional[str] = None,
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> Optional[dict[str, Any]]:
    """
    Search OpenAlex authors via the /authors endpoint. Use this to discover Author IDs (A...) and the...

    Parameters
    ----------
    search : str
        Author name search query. Example: "Yann LeCun".
    per_page : int
        Results per page (OpenAlex max 200).
    page : int
        Page number (1-indexed).
    mailto : str
        Optional contact email for OpenAlex polite pool.
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    Optional[dict[str, Any]]
    """
    # Handle mutable defaults to avoid B006 linting error

    # Strip None values so optional parameters don't trigger schema validation errors
    _args = {
        k: v
        for k, v in {
            "search": search,
            "per_page": per_page,
            "page": page,
            "mailto": mailto,
        }.items()
        if v is not None
    }
    return get_shared_client().run_one_function(
        {
            "name": "openalex_search_authors",
            "arguments": _args,
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["openalex_search_authors"]



"""
gwas_get_snps_for_gene

Get all SNPs mapped to a specific gene.
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def gwas_get_snps_for_gene(
    mapped_gene: str,
    size: Optional[int] = None,
    page: Optional[int] = None,
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> dict[str, Any]:
    """
    Get all SNPs mapped to a specific gene.

    Parameters
    ----------
    mapped_gene : str
        Gene name or symbol
    size : int
        Number of results to return per page
    page : int
        Page number for pagination
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    dict[str, Any]
    """
    # Handle mutable defaults to avoid B006 linting error

    # Strip None values so optional parameters don't trigger schema validation errors
    _args = {
        k: v
        for k, v in {"mapped_gene": mapped_gene, "size": size, "page": page}.items()
        if v is not None
    }
    return get_shared_client().run_one_function(
        {
            "name": "gwas_get_snps_for_gene",
            "arguments": _args,
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["gwas_get_snps_for_gene"]



"""
ENCODE_search_experiments

Search ENCODE functional genomics experiments (e.g., ChIP-seq, ATAC-seq, RNA-seq) by assay type, ...
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def ENCODE_search_experiments(
    assay_title: Optional[str] = None,
    target: Optional[str] = None,
    organism: Optional[str] = None,
    status: Optional[str] = "released",
    limit: Optional[int] = 10,
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> dict[str, Any]:
    """
    Search ENCODE functional genomics experiments (e.g., ChIP-seq, ATAC-seq, RNA-seq) by assay type, ...

    Parameters
    ----------
    assay_title : str
        Assay name filter (e.g., 'ChIP-seq', 'ATAC-seq', 'RNA-seq', 'Hi-C'). Leave em...
    target : str
        Target protein/factor filter (e.g., 'CTCF', 'H3K4me3', 'POLR2A'). Use for ChI...
    organism : str
        Organism filter (e.g., 'Homo sapiens', 'Mus musculus', 'Drosophila melanogast...
    status : str
        Record status filter. Use 'released' for public data (default), 'archived' fo...
    limit : int
        Maximum number of results to return (1–100).
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    dict[str, Any]
    """
    # Handle mutable defaults to avoid B006 linting error

    # Strip None values so optional parameters don't trigger schema validation errors
    _args = {
        k: v
        for k, v in {
            "assay_title": assay_title,
            "target": target,
            "organism": organism,
            "status": status,
            "limit": limit,
        }.items()
        if v is not None
    }
    return get_shared_client().run_one_function(
        {
            "name": "ENCODE_search_experiments",
            "arguments": _args,
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["ENCODE_search_experiments"]



"""
cBioPortal_get_gene_panels

Get all gene panels used in cBioPortal studies. Gene panels define which genes were sequenced in ...
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def cBioPortal_get_gene_panels(
    page_size: Optional[int] = 50,
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> list[Any]:
    """
    Get all gene panels used in cBioPortal studies. Gene panels define which genes were sequenced in ...

    Parameters
    ----------
    page_size : int
        Number of panels to return
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    list[Any]
    """
    # Handle mutable defaults to avoid B006 linting error

    # Strip None values so optional parameters don't trigger schema validation errors
    _args = {k: v for k, v in {"page_size": page_size}.items() if v is not None}
    return get_shared_client().run_one_function(
        {
            "name": "cBioPortal_get_gene_panels",
            "arguments": _args,
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["cBioPortal_get_gene_panels"]



"""
OpenML_get_dataset

Get detailed metadata for a specific OpenML dataset by its numeric ID. Returns comprehensive info...
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def OpenML_get_dataset(
    data_id: int,
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> Any:
    """
    Get detailed metadata for a specific OpenML dataset by its numeric ID. Returns comprehensive info...

    Parameters
    ----------
    data_id : int
        OpenML dataset ID number (e.g., 61 for iris, 554 for MNIST, 31 for credit-g)....
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    Any
    """
    # Handle mutable defaults to avoid B006 linting error

    # Strip None values so optional parameters don't trigger schema validation errors
    _args = {k: v for k, v in {"data_id": data_id}.items() if v is not None}
    return get_shared_client().run_one_function(
        {
            "name": "OpenML_get_dataset",
            "arguments": _args,
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["OpenML_get_dataset"]



"""
GeneNetwork_list_groups

List genetic cross populations (groups) available in GeneNetwork for a given species. These group...
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def GeneNetwork_list_groups(
    species: str,
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> Any:
    """
    List genetic cross populations (groups) available in GeneNetwork for a given species. These group...

    Parameters
    ----------
    species : str
        Species short name (e.g., 'mouse', 'rat', 'human', 'arabidopsis'). Get from G...
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    Any
    """
    # Handle mutable defaults to avoid B006 linting error

    # Strip None values so optional parameters don't trigger schema validation errors
    _args = {k: v for k, v in {"species": species}.items() if v is not None}
    return get_shared_client().run_one_function(
        {
            "name": "GeneNetwork_list_groups",
            "arguments": _args,
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["GeneNetwork_list_groups"]



"""
BioSamples_search_by_filter

Search the EBI BioSamples database with structured attribute filters. More precise than text sear...
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def BioSamples_search_by_filter(
    attribute: str,
    value: str,
    limit: Optional[int | Any] = None,
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> Any:
    """
    Search the EBI BioSamples database with structured attribute filters. More precise than text sear...

    Parameters
    ----------
    attribute : str
        Attribute name to filter on. Common attributes: 'organism', 'tissue', 'diseas...
    value : str
        Value to match for the attribute. Examples: 'Homo sapiens', 'liver', 'melanom...
    limit : int | Any
        Maximum results to return (1-50, default 10).
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    Any
    """
    # Handle mutable defaults to avoid B006 linting error

    # Strip None values so optional parameters don't trigger schema validation errors
    _args = {
        k: v
        for k, v in {"attribute": attribute, "value": value, "limit": limit}.items()
        if v is not None
    }
    return get_shared_client().run_one_function(
        {
            "name": "BioSamples_search_by_filter",
            "arguments": _args,
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["BioSamples_search_by_filter"]



"""
Orphanet API tool for ToolUniverse.

Orphanet is the reference portal for rare diseases and orphan drugs.
This tool uses the Orphadata API and RDcode API for programmatic access.

API Documentation:
- Orphadata: https://api.orphadata.com/
- RDcode: https://api.orphacode.org/
"""

import requests
import urllib.parse
from typing import Dict, Any, Optional, List
from .base_tool import BaseTool
from .tool_registry import register_tool

# Base URLs for Orphanet APIs
ORPHADATA_API_URL = "https://api.orphadata.com"
RDCODE_API_URL = "https://api.orphacode.org"

# RDcode API requires apiKey header (any value accepted)
RDCODE_API_KEY = "ToolUniverse"


def get_rdcode_headers():
    """Get headers for RDcode API requests."""
    return {
        "Accept": "application/json",
        "User-Agent": "ToolUniverse/Orphanet",
        "apiKey": RDCODE_API_KEY,
    }


def normalize_lang(lang: str) -> str:
    """Convert language code to uppercase as required by RDcode API."""
    return lang.upper() if lang else "EN"


@register_tool("OrphanetTool")
class OrphanetTool(BaseTool):
    """
    Tool for querying Orphanet rare disease database.

    Orphanet provides:
    - Rare disease nomenclature and classification
    - Disease-gene associations
    - Epidemiology data (prevalence, inheritance)
    - Expert centers and patient organizations

    RDcode API requires apiKey header (any value accepted).
    Orphadata API is free public access.
    """

    def __init__(self, tool_config: Dict[str, Any]):
        super().__init__(tool_config)
        self.timeout: int = tool_config.get("timeout", 30)
        self.parameter = tool_config.get("parameter", {})

    def run(self, arguments: Dict[str, Any]) -> Dict[str, Any]:
        """Execute Orphanet API call based on operation type."""
        operation = arguments.get("operation", "")

        if operation == "search_diseases":
            return self._search_diseases(arguments)
        elif operation == "get_disease":
            return self._get_disease(arguments)
        elif operation == "get_genes":
            return self._get_genes(arguments)
        elif operation == "get_classification":
            return self._get_classification(arguments)
        elif operation == "search_by_name":
            return self._search_by_name(arguments)
        elif operation == "get_phenotypes":
            return self._get_phenotypes(arguments)
        elif operation == "get_epidemiology":
            return self._get_epidemiology(arguments)
        elif operation == "get_natural_history":
            return self._get_natural_history(arguments)
        elif operation == "get_gene_diseases":
            return self._get_gene_diseases(arguments)
        elif operation == "get_icd_mapping":
            return self._get_icd_mapping(arguments)
        else:
            return {
                "status": "error",
                "error": f"Unknown operation: {operation}. Supported: search_diseases, get_disease, get_genes, get_classification, search_by_name, get_phenotypes, get_epidemiology, get_natural_history, get_gene_diseases, get_icd_mapping",
            }

    def _search_diseases(self, arguments: Dict[str, Any]) -> Dict[str, Any]:
        """
        Search Orphanet for rare diseases by query term.

        Args:
            arguments: Dict containing:
                - query: Search query (disease name, keyword)
                - lang: Language code (en, fr, de, etc.). Default: en
        """
        query = arguments.get("query", "")
        if not query:
            return {"status": "error", "error": "Missing required parameter: query"}

        lang = normalize_lang(arguments.get("lang", "en"))

        try:
            # Use RDcode API for approximate name search
            # URL pattern: /{lang}/ClinicalEntity/ApproximateName/{label}
            encoded_query = urllib.parse.quote(query, safe="")
            response = requests.get(
                f"{RDCODE_API_URL}/{lang}/ClinicalEntity/ApproximateName/{encoded_query}",
                timeout=self.timeout,
                headers=get_rdcode_headers(),
            )
            response.raise_for_status()
            data = response.json()

            # Parse results from API response
            results = []
            if isinstance(data, list):
                results = data
            elif isinstance(data, dict):
                # API returns a dict with entities
                results = data.get("entities", data.get("results", [data]))

            return {
                "status": "success",
                "data": {
                    "results": results,
                    "query": query,
                    "language": lang,
                },
                "metadata": {
                    "source": "Orphanet RDcode API",
                    "query": query,
                },
            }

        except requests.exceptions.HTTPError as e:
            if e.response.status_code == 404:
                return {
                    "status": "success",
                    "data": {"results": [], "query": query, "language": lang},
                    "metadata": {"note": "No diseases found matching the query"},
                }
            return {"status": "error", "error": f"HTTP error: {e.response.status_code}"}
        except requests.exceptions.RequestException as e:
            return {"status": "error", "error": f"Request failed: {str(e)}"}
        except Exception as e:
            return {"status": "error", "error": f"Unexpected error: {str(e)}"}

    def _get_disease(self, arguments: Dict[str, Any]) -> Dict[str, Any]:
        """
        Get disease details by ORPHA code.

        Args:
            arguments: Dict containing:
                - orpha_code: Orphanet disease code (e.g., 558, 166024)
                - lang: Language code (default: en)
        """
        orpha_code = arguments.get("orpha_code", "")
        if not orpha_code:
            return {
                "status": "error",
                "error": "Missing required parameter: orpha_code",
            }

        # Clean ORPHA code
        orpha_code = (
            str(orpha_code).replace("ORPHA:", "").replace("Orphanet:", "").strip()
        )
        lang = normalize_lang(arguments.get("lang", "en"))
        headers = get_rdcode_headers()

        result_data = {"ORPHAcode": orpha_code}

        try:
            # Get preferred name: /{lang}/ClinicalEntity/orphacode/{orphacode}/Name
            name_response = requests.get(
                f"{RDCODE_API_URL}/{lang}/ClinicalEntity/orphacode/{orpha_code}/Name",
                timeout=self.timeout,
                headers=headers,
            )
            name_response.raise_for_status()
            name_data = name_response.json()
            if isinstance(name_data, dict):
                result_data["Preferred term"] = name_data.get(
                    "Preferred term", name_data.get("Name", "")
                )
            else:
                result_data["Preferred term"] = str(name_data) if name_data else ""

        except requests.exceptions.HTTPError as e:
            if e.response.status_code == 404:
                return {
                    "status": "error",
                    "error": f"Disease not found: ORPHA:{orpha_code}",
                }
            return {"status": "error", "error": f"HTTP error: {e.response.status_code}"}

        try:
            # Get definition: /{lang}/ClinicalEntity/orphacode/{orphacode}/Definition
            def_response = requests.get(
                f"{RDCODE_API_URL}/{lang}/ClinicalEntity/orphacode/{orpha_code}/Definition",
                timeout=self.timeout,
                headers=headers,
            )
            if def_response.status_code == 200:
                def_data = def_response.json()
                if isinstance(def_data, dict):
                    result_data["Definition"] = def_data.get("Definition", "")
                else:
                    result_data["Definition"] = str(def_data) if def_data else ""
        except Exception:
            result_data["Definition"] = ""

        try:
            # Get synonyms: /{lang}/ClinicalEntity/orphacode/{orphacode}/Synonym
            syn_response = requests.get(
                f"{RDCODE_API_URL}/{lang}/ClinicalEntity/orphacode/{orpha_code}/Synonym",
                timeout=self.timeout,
                headers=headers,
            )
            if syn_response.status_code == 200:
                syn_data = syn_response.json()
                if isinstance(syn_data, list):
                    result_data["Synonyms"] = syn_data
                elif isinstance(syn_data, dict):
                    result_data["Synonyms"] = syn_data.get(
                        "Synonyms", syn_data.get("Synonym", [])
                    )
                else:
                    result_data["Synonyms"] = []
        except Exception:
            result_data["Synonyms"] = []

        return {
            "status": "success",
            "data": result_data,
            "metadata": {
                "source": "Orphanet RDcode API",
                "orpha_code": orpha_code,
            },
        }

    def _get_genes(self, arguments: Dict[str, Any]) -> Dict[str, Any]:
        """
        Get genes associated with a rare disease.

        Uses Orphadata rd-associated-genes gene name search, since the
        per-orphacode gene endpoint is not reliably available.
        Falls back to RDcode OMIM cross-references to identify associated genes.

        Args:
            arguments: Dict containing:
                - orpha_code: Orphanet disease code
        """
        orpha_code = arguments.get("orpha_code", "")
        if not orpha_code:
            return {
                "status": "error",
                "error": "Missing required parameter: orpha_code",
            }

        orpha_code = (
            str(orpha_code).replace("ORPHA:", "").replace("Orphanet:", "").strip()
        )

        try:
            # First get the disease name from RDcode API
            name_response = requests.get(
                f"{RDCODE_API_URL}/EN/ClinicalEntity/orphacode/{orpha_code}/Name",
                timeout=self.timeout,
                headers=get_rdcode_headers(),
            )
            name_response.raise_for_status()
            name_data = name_response.json()
            disease_name = ""
            if isinstance(name_data, dict):
                disease_name = name_data.get(
                    "Preferred term", name_data.get("Name", "")
                )

            # Search Orphadata gene associations using disease name keywords
            genes = []
            if disease_name:
                # Use first significant word of the disease name to search
                search_term = disease_name.split()[0].lower() if disease_name else ""
                if len(search_term) < 4:
                    # Use longer terms if first word is too short
                    words = [
                        w
                        for w in disease_name.split()
                        if len(w) >= 4
                        and w.lower() not in ("syndrome", "disease", "disorder", "type")
                    ]
                    search_term = (
                        words[0].lower() if words else disease_name.split()[0].lower()
                    )

                try:
                    gene_response = requests.get(
                        f"{ORPHADATA_API_URL}/rd-associated-genes/genes/names/{urllib.parse.quote(search_term, safe='')}",
                        timeout=self.timeout,
                        headers={
                            "Accept": "application/json",
                            "User-Agent": "ToolUniverse/Orphanet",
                        },
                    )
                    if gene_response.status_code == 200:
                        gene_data = gene_response.json()
                        results = gene_data.get("data", {}).get("results", [])
                        # Filter results for our specific ORPHA code
                        for result in results:
                            if str(result.get("ORPHAcode", "")) == str(orpha_code):
                                for assoc in result.get("DisorderGeneAssociation", []):
                                    gene_info = assoc.get("Gene", {})
                                    genes.append(
                                        {
                                            "Symbol": gene_info.get("Symbol", ""),
                                            "Name": gene_info.get("name", ""),
                                            "GeneType": gene_info.get("GeneType", ""),
                                            "Locus": gene_info.get("Locus", []),
                                            "AssociationType": assoc.get(
                                                "DisorderGeneAssociationType", ""
                                            ),
                                            "AssociationStatus": assoc.get(
                                                "DisorderGeneAssociationStatus", ""
                                            ),
                                            "SourceOfValidation": assoc.get(
                                                "SourceOfValidation", ""
                                            ),
                                        }
                                    )
                except Exception:
                    pass

            return {
                "status": "success",
                "data": {
                    "orpha_code": orpha_code,
                    "disease_name": disease_name,
                    "genes": genes,
                },
                "metadata": {
                    "source": "Orphanet Orphadata API",
                    "orpha_code": orpha_code,
                },
            }

        except requests.exceptions.HTTPError as e:
            if e.response.status_code == 404:
                return {
                    "status": "success",
                    "data": {"orpha_code": orpha_code, "disease_name": "", "genes": []},
                    "metadata": {"note": "No gene associations found"},
                }
            return {"status": "error", "error": f"HTTP error: {e.response.status_code}"}
        except requests.exceptions.RequestException as e:
            return {"status": "error", "error": f"Request failed: {str(e)}"}
        except Exception as e:
            return {"status": "error", "error": f"Unexpected error: {str(e)}"}

    def _get_classification(self, arguments: Dict[str, Any]) -> Dict[str, Any]:
        """
        Get disease classification hierarchy.

        Args:
            arguments: Dict containing:
                - orpha_code: Orphanet disease code
                - lang: Language code (default: en)
        """
        orpha_code = arguments.get("orpha_code", "")
        if not orpha_code:
            return {
                "status": "error",
                "error": "Missing required parameter: orpha_code",
            }

        orpha_code = (
            str(orpha_code).replace("ORPHA:", "").replace("Orphanet:", "").strip()
        )
        lang = normalize_lang(arguments.get("lang", "en"))

        try:
            # Use RDcode API: /{lang}/ClinicalEntity/orphacode/{orphacode}/Classification
            response = requests.get(
                f"{RDCODE_API_URL}/{lang}/ClinicalEntity/orphacode/{orpha_code}/Classification",
                timeout=self.timeout,
                headers=get_rdcode_headers(),
            )
            response.raise_for_status()
            data = response.json()

            return {
                "status": "success",
                "data": {
                    "orpha_code": orpha_code,
                    "classification": data,
                },
                "metadata": {
                    "source": "Orphanet RDcode API",
                    "orpha_code": orpha_code,
                },
            }

        except requests.exceptions.HTTPError as e:
            if e.response.status_code == 404:
                return {
                    "status": "success",
                    "data": {"orpha_code": orpha_code, "classification": []},
                    "metadata": {"note": "No classification found"},
                }
            return {"status": "error", "error": f"HTTP error: {e.response.status_code}"}
        except requests.exceptions.RequestException as e:
            return {"status": "error", "error": f"Request failed: {str(e)}"}
        except Exception as e:
            return {"status": "error", "error": f"Unexpected error: {str(e)}"}

    def _search_by_name(self, arguments: Dict[str, Any]) -> Dict[str, Any]:
        """
        Search for diseases by exact or partial name match.

        Args:
            arguments: Dict containing:
                - name: Disease name to search
                - exact: Whether to match exactly (default: False)
                - lang: Language code (default: en)
        """
        name = arguments.get("name", "")
        if not name:
            return {"status": "error", "error": "Missing required parameter: name"}

        exact = arguments.get("exact", False)
        lang = normalize_lang(arguments.get("lang", "en"))

        try:
            # URL encode the search name
            encoded_name = urllib.parse.quote(name, safe="")

            if exact:
                # For exact match, use FindbyName endpoint
                endpoint = (
                    f"{RDCODE_API_URL}/{lang}/ClinicalEntity/FindbyName/{encoded_name}"
                )
            else:
                # For partial match, use ApproximateName endpoint
                endpoint = f"{RDCODE_API_URL}/{lang}/ClinicalEntity/ApproximateName/{encoded_name}"

            response = requests.get(
                endpoint,
                timeout=self.timeout,
                headers=get_rdcode_headers(),
            )
            response.raise_for_status()
            data = response.json()

            # Parse results from response
            if isinstance(data, list):
                results = data
            elif isinstance(data, dict):
                # Single result or wrapped in dict
                results = data.get("entities", data.get("results", [data]))
            else:
                results = [data] if data else []

            return {
                "status": "success",
                "data": {
                    "results": results,
                    "count": len(results),
                    "search_name": name,
                    "exact_match": exact,
                },
                "metadata": {
                    "source": "Orphanet RDcode API",
                    "name": name,
                },
            }

        except requests.exceptions.HTTPError as e:
            if e.response.status_code == 404:
                return {
                    "status": "success",
                    "data": {
                        "results": [],
                        "count": 0,
                        "search_name": name,
                        "exact_match": exact,
                    },
                    "metadata": {"note": "No diseases found matching the name"},
                }
            return {"status": "error", "error": f"HTTP error: {e.response.status_code}"}
        except requests.exceptions.RequestException as e:
            return {"status": "error", "error": f"Request failed: {str(e)}"}
        except Exception as e:
            return {"status": "error", "error": f"Unexpected error: {str(e)}"}

    def _get_phenotypes(self, arguments: Dict[str, Any]) -> Dict[str, Any]:
        """
        Get HPO phenotypes associated with a rare disease.

        Args:
            arguments: Dict containing:
                - orpha_code: Orphanet disease code (e.g., 558 for Marfan)
        """
        orpha_code = arguments.get("orpha_code", "")
        if not orpha_code:
            return {
                "status": "error",
                "error": "Missing required parameter: orpha_code",
            }

        orpha_code = (
            str(orpha_code).replace("ORPHA:", "").replace("Orphanet:", "").strip()
        )

        try:
            response = requests.get(
                f"{ORPHADATA_API_URL}/rd-phenotypes/orphacodes/{orpha_code}",
                timeout=self.timeout,
                headers={
                    "Accept": "application/json",
                    "User-Agent": "ToolUniverse/Orphanet",
                },
            )
            response.raise_for_status()
            data = response.json()

            results = data.get("data", {}).get("results", {})
            disorder = results.get("Disorder", results)
            hpo_associations = disorder.get("HPODisorderAssociation", [])

            phenotypes = []
            for assoc in hpo_associations:
                hpo = assoc.get("HPO", {})
                phenotypes.append(
                    {
                        "hpo_id": hpo.get("HPOId", ""),
                        "hpo_term": hpo.get("HPOTerm", ""),
                        "frequency": assoc.get("HPOFrequency", ""),
                        "diagnostic_criteria": assoc.get("DiagnosticCriteria"),
                    }
                )

            return {
                "status": "success",
                "data": {
                    "orpha_code": orpha_code,
                    "preferred_term": results.get("Preferred term", ""),
                    "phenotypes": phenotypes,
                    "phenotype_count": len(phenotypes),
                },
                "metadata": {
                    "source": "Orphanet Orphadata API",
                    "orpha_code": orpha_code,
                },
            }

        except requests.exceptions.HTTPError as e:
            if e.response.status_code == 404:
                return {
                    "status": "success",
                    "data": {
                        "orpha_code": orpha_code,
                        "preferred_term": "",
                        "phenotypes": [],
                        "phenotype_count": 0,
                    },
                    "metadata": {"note": "No phenotype data found"},
                }
            return {"status": "error", "error": f"HTTP error: {e.response.status_code}"}
        except requests.exceptions.RequestException as e:
            return {"status": "error", "error": f"Request failed: {str(e)}"}
        except Exception as e:
            return {"status": "error", "error": f"Unexpected error: {str(e)}"}

    def _get_epidemiology(self, arguments: Dict[str, Any]) -> Dict[str, Any]:
        """
        Get epidemiology data (prevalence) for a rare disease.

        Args:
            arguments: Dict containing:
                - orpha_code: Orphanet disease code
        """
        orpha_code = arguments.get("orpha_code", "")
        if not orpha_code:
            return {
                "status": "error",
                "error": "Missing required parameter: orpha_code",
            }

        orpha_code = (
            str(orpha_code).replace("ORPHA:", "").replace("Orphanet:", "").strip()
        )

        try:
            response = requests.get(
                f"{ORPHADATA_API_URL}/rd-epidemiology/orphacodes/{orpha_code}",
                timeout=self.timeout,
                headers={
                    "Accept": "application/json",
                    "User-Agent": "ToolUniverse/Orphanet",
                },
            )
            response.raise_for_status()
            data = response.json()

            results = data.get("data", {}).get("results", {})
            prevalence_data = results.get("Prevalence", [])

            prevalences = []
            for p in prevalence_data:
                prevalences.append(
                    {
                        "type": p.get("PrevalenceType", ""),
                        "class": p.get("PrevalenceClass", ""),
                        "geographic": p.get("PrevalenceGeographic", ""),
                        "qualification": p.get("PrevalenceQualification", ""),
                        "mean_value": p.get("ValMoy", ""),
                        "source": p.get("Source", ""),
                        "validation_status": p.get("PrevalenceValidationStatus", ""),
                    }
                )

            return {
                "status": "success",
                "data": {
                    "orpha_code": orpha_code,
                    "preferred_term": results.get("Preferred term", ""),
                    "prevalences": prevalences,
                    "prevalence_count": len(prevalences),
                },
                "metadata": {
                    "source": "Orphanet Orphadata API",
                    "orpha_code": orpha_code,
                },
            }

        except requests.exceptions.HTTPError as e:
            if e.response.status_code == 404:
                return {
                    "status": "success",
                    "data": {
                        "orpha_code": orpha_code,
                        "preferred_term": "",
                        "prevalences": [],
                        "prevalence_count": 0,
                    },
                    "metadata": {"note": "No epidemiology data found"},
                }
            return {"status": "error", "error": f"HTTP error: {e.response.status_code}"}
        except requests.exceptions.RequestException as e:
            return {"status": "error", "error": f"Request failed: {str(e)}"}
        except Exception as e:
            return {"status": "error", "error": f"Unexpected error: {str(e)}"}

    def _get_natural_history(self, arguments: Dict[str, Any]) -> Dict[str, Any]:
        """
        Get natural history data for a rare disease (age of onset, inheritance).

        Args:
            arguments: Dict containing:
                - orpha_code: Orphanet disease code
        """
        orpha_code = arguments.get("orpha_code", "")
        if not orpha_code:
            return {
                "status": "error",
                "error": "Missing required parameter: orpha_code",
            }

        orpha_code = (
            str(orpha_code).replace("ORPHA:", "").replace("Orphanet:", "").strip()
        )

        try:
            response = requests.get(
                f"{ORPHADATA_API_URL}/rd-natural_history/orphacodes/{orpha_code}",
                timeout=self.timeout,
                headers={
                    "Accept": "application/json",
                    "User-Agent": "ToolUniverse/Orphanet",
                },
            )
            response.raise_for_status()
            data = response.json()

            results = data.get("data", {}).get("results", {})

            return {
                "status": "success",
                "data": {
                    "orpha_code": orpha_code,
                    "preferred_term": results.get("Preferred term", ""),
                    "average_age_of_onset": results.get("AverageAgeOfOnset", []),
                    "type_of_inheritance": results.get("TypeOfInheritance", []),
                    "disorder_group": results.get("DisorderGroup", ""),
                    "typology": results.get("Typology", ""),
                },
                "metadata": {
                    "source": "Orphanet Orphadata API",
                    "orpha_code": orpha_code,
                },
            }

        except requests.exceptions.HTTPError as e:
            if e.response.status_code == 404:
                return {
                    "status": "success",
                    "data": {
                        "orpha_code": orpha_code,
                        "preferred_term": "",
                        "average_age_of_onset": [],
                        "type_of_inheritance": [],
                        "disorder_group": "",
                        "typology": "",
                    },
                    "metadata": {"note": "No natural history data found"},
                }
            return {"status": "error", "error": f"HTTP error: {e.response.status_code}"}
        except requests.exceptions.RequestException as e:
            return {"status": "error", "error": f"Request failed: {str(e)}"}
        except Exception as e:
            return {"status": "error", "error": f"Unexpected error: {str(e)}"}

    def _get_gene_diseases(self, arguments: Dict[str, Any]) -> Dict[str, Any]:
        """
        Get rare diseases associated with a gene name.

        Uses Orphadata gene name search to find diseases associated with genes
        matching the given search term. Search by gene name keywords
        (e.g., 'fibrillin', 'huntingtin', 'dystrophin').

        Args:
            arguments: Dict containing:
                - gene_name: Gene name keyword to search (e.g., 'fibrillin', 'collagen')
        """
        gene_name = arguments.get("gene_name", "")
        if not gene_name:
            return {"status": "error", "error": "Missing required parameter: gene_name"}

        try:
            encoded_name = urllib.parse.quote(gene_name, safe="")
            response = requests.get(
                f"{ORPHADATA_API_URL}/rd-associated-genes/genes/names/{encoded_name}",
                timeout=self.timeout,
                headers={
                    "Accept": "application/json",
                    "User-Agent": "ToolUniverse/Orphanet",
                },
            )
            response.raise_for_status()
            data = response.json()

            results = data.get("data", {}).get("results", [])

            diseases = []
            for result in results:
                disease_entry = {
                    "orpha_code": result.get("ORPHAcode", ""),
                    "preferred_term": result.get("Preferred term", ""),
                    "disorder_group": result.get("DisorderGroup", ""),
                    "typology": result.get("Typology", ""),
                    "genes": [],
                }
                for assoc in result.get("DisorderGeneAssociation", []):
                    gene_info = assoc.get("Gene", {})
                    disease_entry["genes"].append(
                        {
                            "symbol": gene_info.get("Symbol", ""),
                            "name": gene_info.get("name", ""),
                            "gene_type": gene_info.get("GeneType", ""),
                            "locus": [
                                loc.get("GeneLocus", "")
                                for loc in gene_info.get("Locus", [])
                            ],
                            "association_type": assoc.get(
                                "DisorderGeneAssociationType", ""
                            ),
                            "association_status": assoc.get(
                                "DisorderGeneAssociationStatus", ""
                            ),
                        }
                    )
                diseases.append(disease_entry)

            return {
                "status": "success",
                "data": {
                    "gene_name": gene_name,
                    "diseases": diseases,
                    "disease_count": len(diseases),
                },
                "metadata": {
                    "source": "Orphanet Orphadata API",
                    "gene_name": gene_name,
                },
            }

        except requests.exceptions.HTTPError as e:
            if e.response.status_code == 404:
                return {
                    "status": "success",
                    "data": {
                        "gene_name": gene_name,
                        "diseases": [],
                        "disease_count": 0,
                    },
                    "metadata": {"note": "No diseases found for this gene name"},
                }
            return {"status": "error", "error": f"HTTP error: {e.response.status_code}"}
        except requests.exceptions.RequestException as e:
            return {"status": "error", "error": f"Request failed: {str(e)}"}
        except Exception as e:
            return {"status": "error", "error": f"Unexpected error: {str(e)}"}

    def _get_icd_mapping(self, arguments: Dict[str, Any]) -> Dict[str, Any]:
        """
        Get ICD-10, ICD-11, OMIM, and SNOMED-CT cross-references for a rare disease.

        Args:
            arguments: Dict containing:
                - orpha_code: Orphanet disease code
                - coding_system: Which coding system to retrieve (icd10, icd11, omim, snomed). Default: all
                - lang: Language code (default: en)
        """
        orpha_code = arguments.get("orpha_code", "")
        if not orpha_code:
            return {
                "status": "error",
                "error": "Missing required parameter: orpha_code",
            }

        orpha_code = (
            str(orpha_code).replace("ORPHA:", "").replace("Orphanet:", "").strip()
        )
        coding_system = arguments.get("coding_system", "all").lower()
        lang = normalize_lang(arguments.get("lang", "en"))
        headers = get_rdcode_headers()

        mappings = {}

        systems_to_query = []
        if coding_system == "all":
            systems_to_query = ["ICD10", "ICD11", "OMIM", "SNOMED-CT"]
        elif coding_system == "icd10":
            systems_to_query = ["ICD10"]
        elif coding_system == "icd11":
            systems_to_query = ["ICD11"]
        elif coding_system == "omim":
            systems_to_query = ["OMIM"]
        elif coding_system == "snomed":
            systems_to_query = ["SNOMED-CT"]
        else:
            return {
                "status": "error",
                "error": f"Unknown coding_system: {coding_system}. Supported: all, icd10, icd11, omim, snomed",
            }

        for system in systems_to_query:
            try:
                response = requests.get(
                    f"{RDCODE_API_URL}/{lang}/ClinicalEntity/orphacode/{orpha_code}/{system}",
                    timeout=self.timeout,
                    headers=headers,
                )
                if response.status_code == 200:
                    data = response.json()
                    if isinstance(data, dict):
                        refs = data.get("References", data.get(f"Code {system}", []))
                        if isinstance(refs, list):
                            mappings[system] = refs
                        elif isinstance(refs, str):
                            mappings[system] = [{"code": refs}]
                        else:
                            mappings[system] = [refs] if refs else []
                    elif isinstance(data, list):
                        mappings[system] = data
                    else:
                        mappings[system] = []
                else:
                    mappings[system] = []
            except Exception:
                mappings[system] = []

        return {
            "status": "success",
            "data": {
                "orpha_code": orpha_code,
                "mappings": mappings,
            },
            "metadata": {
                "source": "Orphanet RDcode API",
                "orpha_code": orpha_code,
                "coding_systems": systems_to_query,
            },
        }



"""
EpiGraphDB_get_drugs_for_trait

Find drugs that target genes associated with a GWAS risk factor trait. Uses genetic evidence to i...
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def EpiGraphDB_get_drugs_for_trait(
    trait: str,
    pval_threshold: Optional[float] = 0.0001,
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> Any:
    """
    Find drugs that target genes associated with a GWAS risk factor trait. Uses genetic evidence to i...

    Parameters
    ----------
    trait : str
        GWAS risk factor trait name (e.g., 'LDL cholesterol', 'Body mass index', 'C-r...
    pval_threshold : float
        GWAS p-value threshold for gene-trait associations (default 1e-4).
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    Any
    """
    # Handle mutable defaults to avoid B006 linting error

    # Strip None values so optional parameters don't trigger schema validation errors
    _args = {
        k: v
        for k, v in {"trait": trait, "pval_threshold": pval_threshold}.items()
        if v is not None
    }
    return get_shared_client().run_one_function(
        {
            "name": "EpiGraphDB_get_drugs_for_trait",
            "arguments": _args,
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["EpiGraphDB_get_drugs_for_trait"]



"""
ChEMBL_get_molecule_image

Get molecular structure image (SVG or PNG format) for a molecule by ChEMBL ID. Returns the image ...
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def ChEMBL_get_molecule_image(
    chembl_id: str,
    format: Optional[str] = "svg",
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> str:
    """
    Get molecular structure image (SVG or PNG format) for a molecule by ChEMBL ID. Returns the image ...

    Parameters
    ----------
    chembl_id : str
        ChEMBL molecule ID (e.g., 'CHEMBL25'). To find a molecule ID, use ChEMBL_sear...
    format : str
        Image format
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    str
    """
    # Handle mutable defaults to avoid B006 linting error

    # Strip None values so optional parameters don't trigger schema validation errors
    _args = {
        k: v
        for k, v in {"chembl_id": chembl_id, "format": format}.items()
        if v is not None
    }
    return get_shared_client().run_one_function(
        {
            "name": "ChEMBL_get_molecule_image",
            "arguments": _args,
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["ChEMBL_get_molecule_image"]



"""
PubChemBioAssay_search_by_gene

Search for PubChem BioAssays that target a specific gene. Returns the list of assay IDs targeting...
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def PubChemBioAssay_search_by_gene(
    gene_symbol: str,
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> Any:
    """
    Search for PubChem BioAssays that target a specific gene. Returns the list of assay IDs targeting...

    Parameters
    ----------
    gene_symbol : str
        Human gene symbol. Examples: 'TP53' (tumor protein p53), 'EGFR' (epidermal gr...
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    Any
    """
    # Handle mutable defaults to avoid B006 linting error

    # Strip None values so optional parameters don't trigger schema validation errors
    _args = {k: v for k, v in {"gene_symbol": gene_symbol}.items() if v is not None}
    return get_shared_client().run_one_function(
        {
            "name": "PubChemBioAssay_search_by_gene",
            "arguments": _args,
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["PubChemBioAssay_search_by_gene"]



"""
BVBRC_search_genome_features

Search for genome features (genes, proteins, CDS) in BV-BRC pathogen genomes. Find specific genes...
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def BVBRC_search_genome_features(
    gene: Optional[str | Any] = None,
    product: Optional[str | Any] = None,
    genome_id: Optional[str | Any] = None,
    limit: Optional[int | Any] = None,
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> Any:
    """
    Search for genome features (genes, proteins, CDS) in BV-BRC pathogen genomes. Find specific genes...

    Parameters
    ----------
    gene : str | Any
        Gene name to search for. Examples: 'mecA' (methicillin resistance), 'blaTEM' ...
    product : str | Any
        Gene product keyword to search. Examples: 'beta-lactamase', 'penicillin-bindi...
    genome_id : str | Any
        Restrict search to a specific genome. Example: '83332.12'.
    limit : int | Any
        Maximum number of results. Default: 10. Max: 100.
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    Any
    """
    # Handle mutable defaults to avoid B006 linting error

    # Strip None values so optional parameters don't trigger schema validation errors
    _args = {
        k: v
        for k, v in {
            "gene": gene,
            "product": product,
            "genome_id": genome_id,
            "limit": limit,
        }.items()
        if v is not None
    }
    return get_shared_client().run_one_function(
        {
            "name": "BVBRC_search_genome_features",
            "arguments": _args,
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["BVBRC_search_genome_features"]



"""
EuropePMC_Guidelines_Search

Search Europe PMC for clinical guidelines and evidence-based recommendations. Europe PMC provides...
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def EuropePMC_Guidelines_Search(
    query: str,
    limit: int,
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> Any:
    """
    Search Europe PMC for clinical guidelines and evidence-based recommendations. Europe PMC provides...

    Parameters
    ----------
    query : str
        Medical condition, treatment, or clinical topic to search for (e.g., 'diabete...
    limit : int
        Maximum number of guidelines to return (default: 10)
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    Any
    """
    # Handle mutable defaults to avoid B006 linting error

    # Strip None values so optional parameters don't trigger schema validation errors
    _args = {k: v for k, v in {"query": query, "limit": limit}.items() if v is not None}
    return get_shared_client().run_one_function(
        {
            "name": "EuropePMC_Guidelines_Search",
            "arguments": _args,
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["EuropePMC_Guidelines_Search"]



"""
get_gget_info

Get comprehensive information about gget – genomics command-line tool and Python package
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def get_gget_info(
    include_examples: bool,
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> dict[str, Any]:
    """
    Get comprehensive information about gget – genomics command-line tool and Python package

    Parameters
    ----------
    include_examples : bool
        Whether to include usage examples and quick start guide
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    dict[str, Any]
    """
    # Handle mutable defaults to avoid B006 linting error

    # Strip None values so optional parameters don't trigger schema validation errors
    _args = {
        k: v for k, v in {"include_examples": include_examples}.items() if v is not None
    }
    return get_shared_client().run_one_function(
        {
            "name": "get_gget_info",
            "arguments": _args,
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["get_gget_info"]



"""
Dataverse_get_dataset

Get detailed metadata for a specific Harvard Dataverse dataset by its persistent DOI identifier. ...
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def Dataverse_get_dataset(
    persistentId: str,
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> Any:
    """
    Get detailed metadata for a specific Harvard Dataverse dataset by its persistent DOI identifier. ...

    Parameters
    ----------
    persistentId : str
        Persistent DOI identifier for the dataset (e.g., 'doi:10.7910/DVN/DUWBBU', 'd...
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    Any
    """
    # Handle mutable defaults to avoid B006 linting error

    # Strip None values so optional parameters don't trigger schema validation errors
    _args = {k: v for k, v in {"persistentId": persistentId}.items() if v is not None}
    return get_shared_client().run_one_function(
        {
            "name": "Dataverse_get_dataset",
            "arguments": _args,
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["Dataverse_get_dataset"]



"""
PDC_get_gene_protein

Get protein information and proteomics study coverage for a gene from the NCI Proteomics Data Com...
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def PDC_get_gene_protein(
    operation: str,
    gene_name: str,
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> dict[str, Any]:
    """
    Get protein information and proteomics study coverage for a gene from the NCI Proteomics Data Com...

    Parameters
    ----------
    operation : str
        Operation type
    gene_name : str
        Gene symbol to look up (e.g., 'TP53', 'EGFR', 'BRCA1', 'MYC')
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    dict[str, Any]
    """
    # Handle mutable defaults to avoid B006 linting error

    # Strip None values so optional parameters don't trigger schema validation errors
    _args = {
        k: v
        for k, v in {"operation": operation, "gene_name": gene_name}.items()
        if v is not None
    }
    return get_shared_client().run_one_function(
        {
            "name": "PDC_get_gene_protein",
            "arguments": _args,
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["PDC_get_gene_protein"]



"""
gnomad_get_sv_by_region

Get structural variants (SVs) from gnomAD v4 overlapping a genomic region. Returns DEL/DUP/INV/BN...
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def gnomad_get_sv_by_region(
    chrom: str,
    start: int,
    stop: int,
    reference_genome: Optional[str] = "GRCh38",
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> dict[str, Any]:
    """
    Get structural variants (SVs) from gnomAD v4 overlapping a genomic region. Returns DEL/DUP/INV/BN...

    Parameters
    ----------
    chrom : str
        Chromosome (e.g., '17', 'X').
    start : int
        Start position (1-based, GRCh38 by default).
    stop : int
        Stop position.
    reference_genome : str
        Reference genome assembly.
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    dict[str, Any]
    """
    # Handle mutable defaults to avoid B006 linting error

    # Strip None values so optional parameters don't trigger schema validation errors
    _args = {
        k: v
        for k, v in {
            "chrom": chrom,
            "start": start,
            "stop": stop,
            "reference_genome": reference_genome,
        }.items()
        if v is not None
    }
    return get_shared_client().run_one_function(
        {
            "name": "gnomad_get_sv_by_region",
            "arguments": _args,
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["gnomad_get_sv_by_region"]



"""
EBITaxonomy_get_by_scientific_name

Look up taxonomy by scientific (Latin) name using the EBI Taxonomy service. Returns taxonomy ID, ...
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def EBITaxonomy_get_by_scientific_name(
    scientific_name: str,
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> Any:
    """
    Look up taxonomy by scientific (Latin) name using the EBI Taxonomy service. Returns taxonomy ID, ...

    Parameters
    ----------
    scientific_name : str
        Scientific name of the organism. Examples: 'Homo sapiens', 'Escherichia coli'...
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    Any
    """
    # Handle mutable defaults to avoid B006 linting error

    # Strip None values so optional parameters don't trigger schema validation errors
    _args = {
        k: v for k, v in {"scientific_name": scientific_name}.items() if v is not None
    }
    return get_shared_client().run_one_function(
        {
            "name": "EBITaxonomy_get_by_scientific_name",
            "arguments": _args,
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["EBITaxonomy_get_by_scientific_name"]



"""
GTDB_get_species

Get detailed information about a GTDB species cluster, including all genomes assigned to that spe...
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def GTDB_get_species(
    operation: str,
    species: str,
    max_genomes: Optional[int] = None,
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> Any:
    """
    Get detailed information about a GTDB species cluster, including all genomes assigned to that spe...

    Parameters
    ----------
    operation : str
        Operation type (fixed: get_species)
    species : str
        Species name WITHOUT the s__ prefix (e.g., 'Escherichia coli', 'Hydrogenother...
    max_genomes : int
        Maximum number of genomes to return (default: 20). Species clusters can have ...
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    Any
    """
    # Handle mutable defaults to avoid B006 linting error

    # Strip None values so optional parameters don't trigger schema validation errors
    _args = {
        k: v
        for k, v in {
            "operation": operation,
            "species": species,
            "max_genomes": max_genomes,
        }.items()
        if v is not None
    }
    return get_shared_client().run_one_function(
        {
            "name": "GTDB_get_species",
            "arguments": _args,
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["GTDB_get_species"]



"""
PharmVar_get_alleles

Get all allele definitions for a pharmacogene from PharmVar (Pharmacogene Variation Consortium). ...
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def PharmVar_get_alleles(
    gene: str,
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> Any:
    """
    Get all allele definitions for a pharmacogene from PharmVar (Pharmacogene Variation Consortium). ...

    Parameters
    ----------
    gene : str
        Gene symbol (e.g., 'CYP2D6', 'CYP2C9', 'CYP2C19', 'CYP2B6', 'DPYD', 'SLCO1B1'...
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    Any
    """
    # Handle mutable defaults to avoid B006 linting error

    # Strip None values so optional parameters don't trigger schema validation errors
    _args = {k: v for k, v in {"gene": gene}.items() if v is not None}
    return get_shared_client().run_one_function(
        {
            "name": "PharmVar_get_alleles",
            "arguments": _args,
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["PharmVar_get_alleles"]



"""
MetaboAnalyst_pathway_enrichment

Perform metabolite pathway enrichment analysis (over-representation analysis) using hypergeometri...
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def MetaboAnalyst_pathway_enrichment(
    metabolites: list[str],
    organism: Optional[str] = None,
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> list[Any]:
    """
    Perform metabolite pathway enrichment analysis (over-representation analysis) using hypergeometri...

    Parameters
    ----------
    metabolites : list[str]
        List of metabolite names to test for pathway enrichment. Example: ['glucose',...
    organism : str
        KEGG organism code. Default 'hsa' (human). Common: hsa=human, mmu=mouse, rno=...
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    list[Any]
    """
    # Handle mutable defaults to avoid B006 linting error

    # Strip None values so optional parameters don't trigger schema validation errors
    _args = {
        k: v
        for k, v in {"metabolites": metabolites, "organism": organism}.items()
        if v is not None
    }
    return get_shared_client().run_one_function(
        {
            "name": "MetaboAnalyst_pathway_enrichment",
            "arguments": _args,
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["MetaboAnalyst_pathway_enrichment"]



"""
FAERS_search_reports_by_drug_and_outcome

Search and retrieve detailed adverse event reports for a specific drug filtered by reaction outco...
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def FAERS_search_reports_by_drug_and_outcome(
    medicinalproduct: str,
    reactionoutcome: Optional[str] = None,
    limit: Optional[int] = 10,
    skip: Optional[int] = 0,
    patientsex: Optional[str] = None,
    patientagegroup: Optional[str] = None,
    serious: Optional[str] = None,
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> list[Any]:
    """
    Search and retrieve detailed adverse event reports for a specific drug filtered by reaction outco...

    Parameters
    ----------
    medicinalproduct : str
        Drug name (required).
    reactionoutcome : str
        Optional: Filter by reaction outcome. Omit this parameter if you don't want t...
    limit : int
        Maximum number of reports to return. Must be between 1 and 100.
    skip : int
        Number of reports to skip for pagination. Must be non-negative.
    patientsex : str
        Optional: Filter by patient sex. Omit this parameter if you don't want to fil...
    patientagegroup : str
        Optional: Filter by patient age group. Omit this parameter if you don't want ...
    serious : str
        Optional: Filter by event seriousness. Omit this parameter if you don't want ...
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    list[Any]
    """
    # Handle mutable defaults to avoid B006 linting error

    # Strip None values so optional parameters don't trigger schema validation errors
    _args = {
        k: v
        for k, v in {
            "medicinalproduct": medicinalproduct,
            "reactionoutcome": reactionoutcome,
            "limit": limit,
            "skip": skip,
            "patientsex": patientsex,
            "patientagegroup": patientagegroup,
            "serious": serious,
        }.items()
        if v is not None
    }
    return get_shared_client().run_one_function(
        {
            "name": "FAERS_search_reports_by_drug_and_outcome",
            "arguments": _args,
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["FAERS_search_reports_by_drug_and_outcome"]



"""
NeuroVault_list_collection_images

List all statistical map images in a specific NeuroVault collection. Returns image metadata inclu...
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def NeuroVault_list_collection_images(
    collection_id: int,
    limit: Optional[int | Any] = None,
    offset: Optional[int | Any] = None,
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> Any:
    """
    List all statistical map images in a specific NeuroVault collection. Returns image metadata inclu...

    Parameters
    ----------
    collection_id : int
        NeuroVault collection ID. Example: 457 (Human Connectome Project)
    limit : int | Any
        Maximum number of images to return (default 10, max 100)
    offset : int | Any
        Number of images to skip for pagination (default 0)
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    Any
    """
    # Handle mutable defaults to avoid B006 linting error

    # Strip None values so optional parameters don't trigger schema validation errors
    _args = {
        k: v
        for k, v in {
            "collection_id": collection_id,
            "limit": limit,
            "offset": offset,
        }.items()
        if v is not None
    }
    return get_shared_client().run_one_function(
        {
            "name": "NeuroVault_list_collection_images",
            "arguments": _args,
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["NeuroVault_list_collection_images"]



"""
NeuroVault_get_image

Get detailed metadata for a specific brain statistical map image in NeuroVault by image ID. Retur...
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def NeuroVault_get_image(
    image_id: int,
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> Any:
    """
    Get detailed metadata for a specific brain statistical map image in NeuroVault by image ID. Retur...

    Parameters
    ----------
    image_id : int
        NeuroVault image ID. Examples: 3128 (HCP emotion task Z map), 946 (pain T map)
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    Any
    """
    # Handle mutable defaults to avoid B006 linting error

    # Strip None values so optional parameters don't trigger schema validation errors
    _args = {k: v for k, v in {"image_id": image_id}.items() if v is not None}
    return get_shared_client().run_one_function(
        {
            "name": "NeuroVault_get_image",
            "arguments": _args,
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["NeuroVault_get_image"]



"""
Zenodo_list_licenses

List available licenses that can be applied to Zenodo uploads. Returns license metadata including...
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def Zenodo_list_licenses(
    query: Optional[str] = None,
    limit: Optional[int] = 25,
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> dict[str, Any]:
    """
    List available licenses that can be applied to Zenodo uploads. Returns license metadata including...

    Parameters
    ----------
    query : str
        Optional search query to filter licenses by name or keyword (e.g., 'creative ...
    limit : int
        Maximum number of licenses to return (default: 25, max: 100).
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    dict[str, Any]
    """
    # Handle mutable defaults to avoid B006 linting error

    # Strip None values so optional parameters don't trigger schema validation errors
    _args = {k: v for k, v in {"query": query, "limit": limit}.items() if v is not None}
    return get_shared_client().run_one_function(
        {
            "name": "Zenodo_list_licenses",
            "arguments": _args,
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["Zenodo_list_licenses"]



"""
Rhea_search_by_chebi

Search for biochemical reactions involving a specific ChEBI compound in Rhea. Returns reactions w...
"""

from typing import Any, Optional, Callable
from ._shared_client import get_shared_client


def Rhea_search_by_chebi(
    chebi_id: str,
    limit: Optional[int | Any] = None,
    *,
    stream_callback: Optional[Callable[[str], None]] = None,
    use_cache: bool = False,
    validate: bool = True,
) -> Any:
    """
    Search for biochemical reactions involving a specific ChEBI compound in Rhea. Returns reactions w...

    Parameters
    ----------
    chebi_id : str
        ChEBI compound identifier. Can include or omit 'CHEBI:' prefix. Examples: 'CH...
    limit : int | Any
        Maximum number of results (default 20, max 50).
    stream_callback : Callable, optional
        Callback for streaming output
    use_cache : bool, default False
        Enable caching
    validate : bool, default True
        Validate parameters

    Returns
    -------
    Any
    """
    # Handle mutable defaults to avoid B006 linting error

    # Strip None values so optional parameters don't trigger schema validation errors
    _args = {
        k: v for k, v in {"chebi_id": chebi_id, "limit": limit}.items() if v is not None
    }
    return get_shared_client().run_one_function(
        {
            "name": "Rhea_search_by_chebi",
            "arguments": _args,
        },
        stream_callback=stream_callback,
        use_cache=use_cache,
        validate=validate,
    )


__all__ = ["Rhea_search_by_chebi"]
